/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.registry.collections;

import com.liferay.registry.Registry;
import com.liferay.registry.RegistryUtil;
import com.liferay.registry.ServiceReference;

/**
 * @author Carlos Sierra Andrés
 */
public class ServiceReferenceMapperFactory {

	public static <K, S> ServiceReferenceMapper<K, S> create(
		final ServiceMapper<K, S> serviceMapper) {

		return new ServiceReferenceMapper<K, S>() {

			@Override
			public void map(
				ServiceReference<S> serviceReference, Emitter<K> emitter) {

				Registry registry = RegistryUtil.getRegistry();

				S service = registry.getService(serviceReference);

				try {
					serviceMapper.map(service, emitter);
				}
				finally {
					registry.ungetService(serviceReference);
				}
			}

		};
	}

	public static <K, S> ServiceReferenceMapper<K, S> create(
		final String propertyKey) {

		return new ServiceReferenceMapper<K, S>() {

			@Override
			public void map(
				ServiceReference<S> serviceReference, Emitter<K> emitter) {

				Object propertyValue = serviceReference.getProperty(
					propertyKey);

				if (propertyValue == null) {
					return;
				}

				if (propertyValue instanceof Object[]) {
					for (K k : (K[])propertyValue) {
						emitter.emit(k);
					}
				}
				else {
					emitter.emit((K)propertyValue);
				}
			}

		};
	}

}