/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.registry.dependency;

import com.liferay.registry.Registry;
import com.liferay.registry.RegistryUtil;
import com.liferay.registry.ServiceReference;

/**
 * @author Michael C. Han
 */
public class ClassServiceDependencyVerifier
	implements ServiceDependencyVerifier {

	public ClassServiceDependencyVerifier(Class<?> clazz) {
		_clazz = clazz;
	}

	@Override
	public boolean equals(Object object) {
		if (this == object) {
			return true;
		}

		if ((object == null) || (getClass() != object.getClass())) {
			return false;
		}

		return _clazz.equals(object);
	}

	@Override
	public int hashCode() {
		return _clazz.hashCode();
	}

	@Override
	public String toString() {
		return _clazz.getName();
	}

	@Override
	public boolean verify(ServiceReference<?> serviceReference) {
		Registry registry = RegistryUtil.getRegistry();

		Object service = registry.getService(serviceReference);

		Class<?> serviceClass = service.getClass();

		if (serviceClass.equals(_clazz)) {
			return true;
		}

		Class<?>[] interfaceClasses = serviceClass.getInterfaces();

		for (Class<?> interfaceClass : interfaceClasses) {
			if (interfaceClass.equals(_clazz)) {
				return true;
			}
		}

		return false;
	}

	private final Class<?> _clazz;

}